
// Variables pour HC-SR04
const int Echo_in_pin = 2; 
const int Trig_out_pin = 3;
float Dist_mm=0.0;

// Vitesse du son (mm/us) 
const double Speed_sound = 340.0e-3;

// Paramètres 
const int LED_r=5;  
const int LED_g=6;  
const int LED_b=7;  

void setup() 
{    
    // Initialisation HC-SR04
    pinMode(Trig_out_pin, OUTPUT);
    pinMode(Echo_in_pin, INPUT);
    digitalWrite(Trig_out_pin, LOW);
  
    // Init interface série 
    Serial.begin(115200); 

    // LEDs 
    pinMode(LED_r,OUTPUT);
    pinMode(LED_g,OUTPUT);
    pinMode(LED_b,OUTPUT);
    digitalWrite(LED_r, LOW);
    digitalWrite(LED_g, LOW);
    digitalWrite(LED_b, LOW);
}

void loop() 
{       

    /* Mesure de la distance */ 
    Dist_mm=GetDisCM( Echo_in_pin,  Trig_out_pin,  Speed_sound); // Valeur brute 
    //Dist_mm=GetMean(8,  Echo_in_pin,  Trig_out_pin,  Speed_sound); // Valeur moyenne 

    /* Affichage de la distance */
    Serial.println(Dist_mm); 
    
    /* Activation des LEDs */
    SetLED(Dist_mm, LED_r, LED_g, LED_b);
}

// Calcul de la distance en mm 
float GetDisCM(int Echo_in_pinn, int Trig_out_pinn, double Speed_soundd)
{
  float Dist_milm=0.0;
  long HC_val=0; 
  
  // Déclanchement mesure avec l'envoie d'une impulsion de 10µs
  digitalWrite(Trig_out_pin, HIGH);
  delayMicroseconds(10);
  digitalWrite(Trig_out_pin, LOW);
  
  // Lecture du temps aller/retour de l'onde 
  HC_val = pulseIn(Echo_in_pin, HIGH, 30000);
    
  // Calcul de distance d(mm), V=d/t==> v*t 
  // Le coefficnet 2 pour la distance aller
  Dist_milm = (HC_val/2.0) * Speed_sound; 

  return Dist_milm;
} 

// Valeur Moyenne 
float GetMean(long Taille, int Echo_in_pinn, int Trig_out_pinn, double Speed_soundd)
{
  long i;
  float Distance_tmp, Mean_val=0.0,Somme=0.0; 
  for(i=0; i<Taille; i++)
  {
    Distance_tmp=GetDisCM( Echo_in_pinn,  Trig_out_pinn,  Speed_soundd);
    Somme=Somme+Distance_tmp; 
  }
  Mean_val=Somme/Taille; 
  return Mean_val;
}

void SetLED(float Dist_mmm, int LED_rr, int LED_gg, int LED_bb)
{
    if((Dist_mmm < 500) && (Dist_mmm != 0.0))
    {
      digitalWrite(LED_rr, HIGH);
      digitalWrite(LED_gg, HIGH);
      digitalWrite(LED_bb, HIGH);
      delay(floor(Dist_mmm));
      digitalWrite(LED_rr, LOW);
      digitalWrite(LED_gg, LOW);
      digitalWrite(LED_bb, LOW);
      delay(floor(Dist_mmm));
    }  
}
